@@ -8,53 +8,66 @@ from datetime import datetime
 from typing import TYPE_CHECKING, Any, Generic, TypeVar
 from uuid import uuid4
 
-from starlite_saqlalchemy.db.orm import Base
+from starlite_saqlalchemy.db import orm
 from starlite_saqlalchemy.repository.abc import AbstractRepository
 from starlite_saqlalchemy.repository.exceptions import RepositoryConflictException
 
 if TYPE_CHECKING:
-    from collections import abc
+    from collections.abc import Callable, Hashable, Iterable, MutableMapping
 
     from starlite_saqlalchemy.repository.types import FilterTypes
 
-BaseT = TypeVar("BaseT", bound=Base)
+ModelT = TypeVar("ModelT", bound=orm.Base)
+MockRepoT = TypeVar("MockRepoT", bound="GenericMockRepository")
 
 
-class GenericMockRepository(AbstractRepository[BaseT], Generic[BaseT]):
+class GenericMockRepository(AbstractRepository[ModelT], Generic[ModelT]):
     """A repository implementation for tests.
 
     Uses a `dict` for storage.
     """
 
-    collection: abc.MutableMapping[abc.Hashable, BaseT] = {}
+    _collections: MutableMapping[type[ModelT], MutableMapping[Hashable, ModelT]] = {}
 
-    def __init__(self, id_factory: abc.Callable[[], Any] = uuid4, **_: Any) -> None:
+    def __init__(self, id_factory: Callable[[], Any] = uuid4, **_: Any) -> None:
         super().__init__()
         self._id_factory = id_factory
+        self.collection = self._collections[self.model_type]
 
-    def _find_or_raise_not_found(self, id_: Any) -> BaseT:
+    @classmethod
+    def __class_getitem__(cls: type[MockRepoT], item: type[ModelT]) -> type[MockRepoT]:
+        """Add collection to `_collections` for the type.
+
+        Args:
+            item: The type that the class has been parametrized with.
+        """
+        cls._collections.setdefault(item, {})
+        return cls
+
+    def _find_or_raise_not_found(self, id_: Any) -> ModelT:
         return self.check_not_found(self.collection.get(id_))
 
-    async def add(self, data: BaseT, _allow_id: bool = False) -> BaseT:
+    async def add(self, data: ModelT, allow_id: bool = False) -> ModelT:
         """Add `data` to the collection.
 
         Args:
             data: Instance to be added to the collection.
+            allow_id: disable the identified object check.
 
         Returns:
             The added instance.
         """
-        if _allow_id is False and self.get_id_attribute_value(data) is not None:
+        if allow_id is False and self.get_id_attribute_value(data) is not None:
             raise RepositoryConflictException("`add()` received identified item.")
         now = datetime.now()
         data.updated = data.created = now
-        if _allow_id is False:
+        if allow_id is False:
             id_ = self._id_factory()
             self.set_id_attribute_value(id_, data)
         self.collection[data.id] = data
         return data
 
-    async def delete(self, id_: Any) -> BaseT:
+    async def delete(self, id_: Any) -> ModelT:
         """Delete instance identified by `id_`.
 
         Args:
@@ -71,7 +84,7 @@ class GenericMockRepository(AbstractRepository[BaseT], Generic[BaseT]):
         finally:
             del self.collection[id_]
 
-    async def get(self, id_: Any) -> BaseT:
+    async def get(self, id_: Any) -> ModelT:
         """Get instance identified by `id_`.
 
         Args:
@@ -85,7 +98,7 @@ class GenericMockRepository(AbstractRepository[BaseT], Generic[BaseT]):
         """
         return self._find_or_raise_not_found(id_)
 
-    async def list(self, *filters: "FilterTypes", **kwargs: Any) -> list[BaseT]:
+    async def list(self, *filters: "FilterTypes", **kwargs: Any) -> list[ModelT]:
         """Get a list of instances, optionally filtered.
 
         Args:
@@ -97,7 +110,7 @@ class GenericMockRepository(AbstractRepository[BaseT], Generic[BaseT]):
         """
         return list(self.collection.values())
 
-    async def update(self, data: BaseT) -> BaseT:
+    async def update(self, data: ModelT) -> ModelT:
         """Update instance with the attribute values present on `data`.
 
         Args:
@@ -119,7 +132,7 @@ class GenericMockRepository(AbstractRepository[BaseT], Generic[BaseT]):
             setattr(item, key, val)
         return item
 
-    async def upsert(self, data: BaseT) -> BaseT:
+    async def upsert(self, data: ModelT) -> ModelT:
         """Update or create instance.
 
         Updates instance with the attribute values present on `data`, or creates a new instance if
@@ -139,4 +152,20 @@ class GenericMockRepository(AbstractRepository[BaseT], Generic[BaseT]):
         id_ = self.get_id_attribute_value(data)
         if id_ in self.collection:
             return await self.update(data)
-        return await self.add(data, _allow_id=True)
+        return await self.add(data, allow_id=True)
+
+    @classmethod
+    def seed_collection(cls, instances: Iterable[ModelT]) -> None:
+        """Seed the collection for repository type.
+
+        Args:
+            instances: the instances to be added to the collection.
+        """
+        collection = cls._collections[cls.model_type]
+        for instance in instances:
+            collection[cls.get_id_attribute_value(instance)] = instance
+
+    @classmethod
+    def clear_collection(cls) -> None:
+        """Empty the collection for repository type."""
+        cls._collections[cls.model_type] = {}
